@@ -107,19 +107,6 @@ anjuta_util_copy_file (gchar * src, gchar * dest, gboolean show_error)
 	return error;
 }
 
-static gint
-int_from_hex_digit (const gchar ch)
-{
-	if (isdigit (ch))
-		return ch - '0';
-	else if (ch >= 'A' && ch <= 'F')
-		return ch - 'A' + 10;
-	else if (ch >= 'a' && ch <= 'f')
-		return ch - 'a' + 10;
-	else
-		return 0;
-}
-
 void
 anjuta_util_color_from_string (const gchar * val, guint16 * r, guint16 * g, guint16 * b)
 {
@@ -133,40 +120,19 @@ anjuta_util_color_from_string (const gchar * val, guint16 * r, guint16 * g, guin
 }
 
 gchar *
-anjuta_util_string_from_color (guint8 r, guint8 g, guint8 b)
+anjuta_util_string_from_color (guint16 r, guint16 g, guint16 b)
 {
-	gchar str[10];
-	guint32 num;
-
-	num = r;
-	num <<= 8;
-	num += g;
-	num <<= 8;
-	num += b;
-
-	sprintf (str, "#%06X", num);
-	return g_strdup (str);
+	return g_strdup_printf("#%02x%02x%02x", r >> 8, g >> 8, b >> 8);
 }
 
 /* Get a GdkColor from preferences. Free the color with gdk_color_free() */
 GdkColor*
 anjuta_util_convert_color(AnjutaPreferences* prefs, const gchar* pref_name)
 {
-	guint16 r, g, b;
-	guint factor = ((guint16) -1) / ((guint8) -1);
-	gchar* color;
-	GdkColor* gdkcolor = g_new0(GdkColor, 1);
-	color = anjuta_preferences_get(prefs, pref_name);
-	if (color)
-	{
-		anjuta_util_color_from_string (color, &r, &g, &b);
-		gdkcolor->pixel = 0;
-		gdkcolor->red = r * factor;
-		gdkcolor->green = g * factor;
-		gdkcolor->blue = b * factor;
-		g_free(color);
-	}
-	return gdkcolor;
+	GdkColor* color = g_new0(GdkColor, 1);
+	gchar* color_string = anjuta_preferences_get(prefs, pref_name);
+	gdk_color_parse(color_string, color);
+	return color;
 }
 
 GtkWidget* 
