@@ -31,6 +31,8 @@
 #include <gtk/gtkfilechooserdialog.h>
 #include <libgnomevfs/gnome-vfs.h>
 
+#include <gdl/gdl-icons.h>
+
 #include "anjuta-docman.h"
 #include "file_history.h"
 #include "plugin.h"
@@ -49,10 +51,12 @@ static gpointer parent_class;
 #define DRAG_AFTER_ALL_TABS -1
 
 struct _AnjutaDocmanPriv {
+	DocmanPlugin *plugin;
 	AnjutaPreferences *preferences;
 	IAnjutaEditor *current_editor;
+	
 	GtkWidget *fileselection;
-	GtkWidget *save_as_fileselection;
+	
 	GList *editors;
 	GtkWidget *popup_menu;
 	gboolean shutingdown;
@@ -73,9 +77,11 @@ struct _AnjutaDocmanPage {
 	GtkWidget *widget;
 	GtkWidget *close_image;
 	GtkWidget *close_button;
+	GtkWidget *mime_icon;
 	GtkWidget *label;
 	GtkWidget *menu_label;
 	GtkWidget *box;
+	gboolean is_current;
 };
 
 typedef struct _AnjutaDocmanPage AnjutaDocmanPage;
@@ -469,14 +475,43 @@ static void
 on_text_editor_notebook_close_page (GtkButton* button, 
 									AnjutaDocman* docman)
 {
-	/* This is a hack because we cannot get the real
-	Editor plugin */
-	DocmanPlugin* dummy_plugin = g_new0(DocmanPlugin, 1);
-	dummy_plugin->docman = GTK_WIDGET(docman);
-
-	on_close_file1_activate (NULL, dummy_plugin);
+	GList* node;
 	
-	g_free(dummy_plugin);
+	node = docman->priv->editors;
+	while (node)
+	{
+		AnjutaDocmanPage *page;
+		IAnjutaEditor *te;
+		page = (AnjutaDocmanPage *) node->data;
+		if (page->close_button == GTK_WIDGET(button))
+		{
+			te = IANJUTA_EDITOR (page->widget);
+			anjuta_docman_set_current_editor(docman, te);
+			break;
+		}
+		node = g_list_next (node);
+	}
+	
+	on_close_file_activate (NULL, docman->priv->plugin);
+}
+
+static void
+on_text_editor_notebook_close_button_enter_cb (GtkButton* button, 
+											   AnjutaDocmanPage* page)
+{
+	g_return_if_fail (page != NULL);
+	
+	gtk_widget_set_sensitive (page->close_image, TRUE);
+}
+
+static void
+on_text_editor_notebook_close_button_leave_cb (GtkButton* button, 
+											   AnjutaDocmanPage* page)
+{
+	g_return_if_fail (page != NULL);
+	
+	if (! page->is_current)
+		gtk_widget_set_sensitive (page->close_image,FALSE);
 }
 
 static void
@@ -487,22 +522,27 @@ editor_tab_widget_destroy (AnjutaDocmanPage* page)
 
 	page->close_image = NULL;
 	page->close_button = NULL;
+	page->mime_icon = NULL;
 	page->label = NULL;
 	page->menu_label = NULL;
+	page->is_current = FALSE;
 }
 
 static GtkWidget*
 editor_tab_widget_new(AnjutaDocmanPage* page, AnjutaDocman* docman)
 {
-	GtkWidget *button15;
+	GtkWidget *close_button;
 	GtkWidget *close_pixmap;
-	GtkWidget *tmp_toolbar_icon;
+	GtkRcStyle *rcstyle;
 	GtkWidget *label, *menu_label;
 	GtkWidget *box;
 	GtkWidget *event_hbox;
 	GtkWidget *event_box;
+	GtkTooltips *tooltips;
 	int h, w;
 	GdkColor color;
+	gchar* uri;
+	IAnjutaFile* editor;
 	
 	g_return_val_if_fail(GTK_IS_WIDGET (page->widget), NULL);
 	
@@ -511,21 +551,25 @@ editor_tab_widget_new(AnjutaDocmanPage* page, AnjutaDocman* docman)
 	
 	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
 
-	tmp_toolbar_icon = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	gtk_widget_show(tmp_toolbar_icon);
-	
-	button15 = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(button15), tmp_toolbar_icon);
-	gtk_button_set_relief(GTK_BUTTON(button15), GTK_RELIEF_NONE);
-	
-	/* Setting size creates awkard looking cross button. Let the widget
-	 * size itself
-	 */
-	/* gtk_widget_set_size_request (button15, w, h); */
-
 	close_pixmap = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	gtk_widget_set_size_request(close_pixmap, w,h);
-	gtk_widget_set_sensitive(close_pixmap, FALSE);
+	gtk_widget_show(close_pixmap);
+	
+	/* setup close button, zero out {x,y}thickness to get smallest possible
+	 * size */
+	close_button = gtk_button_new();
+	gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
+	gtk_container_add(GTK_CONTAINER(close_button), close_pixmap);
+	gtk_button_set_relief(GTK_BUTTON(close_button), GTK_RELIEF_NONE);
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = rcstyle->ythickness = 0;
+	gtk_widget_modify_style (close_button, rcstyle);
+	gtk_rc_style_unref (rcstyle);
+	
+	gtk_widget_set_size_request (close_button, w, h);
+	tooltips = gtk_tooltips_new ();
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), close_button,
+						  _("Close file"),
+						  NULL);
 
 	label = gtk_label_new (ianjuta_editor_get_filename (IANJUTA_EDITOR(page->widget),
 														NULL));
@@ -539,12 +583,12 @@ editor_tab_widget_new(AnjutaDocmanPage* page, AnjutaDocman* docman)
 	color.green = 0;
 	color.blue = 0;
 	
-	gtk_widget_modify_fg (button15, GTK_STATE_NORMAL, &color);
-	gtk_widget_modify_fg (button15, GTK_STATE_INSENSITIVE, &color);
-	gtk_widget_modify_fg (button15, GTK_STATE_ACTIVE, &color);
-	gtk_widget_modify_fg (button15, GTK_STATE_PRELIGHT, &color);
-	gtk_widget_modify_fg (button15, GTK_STATE_SELECTED, &color);
-	gtk_widget_show(button15);
+	gtk_widget_modify_fg (close_button, GTK_STATE_NORMAL, &color);
+	gtk_widget_modify_fg (close_button, GTK_STATE_INSENSITIVE, &color);
+	gtk_widget_modify_fg (close_button, GTK_STATE_ACTIVE, &color);
+	gtk_widget_modify_fg (close_button, GTK_STATE_PRELIGHT, &color);
+	gtk_widget_modify_fg (close_button, GTK_STATE_SELECTED, &color);
+	gtk_widget_show(close_button);
 	
 	/* create our layout/event boxes */
 	event_box = gtk_event_box_new();
@@ -553,9 +597,24 @@ editor_tab_widget_new(AnjutaDocmanPage* page, AnjutaDocman* docman)
 	event_hbox = gtk_hbox_new (FALSE, 2);	
 	box = gtk_hbox_new(FALSE, 2);
 	
+	/* Add a nice mime-type icon */
+	editor = IANJUTA_FILE(page->widget);
+	uri = ianjuta_file_get_uri(editor, NULL);
+	if (uri != NULL)
+	{
+		const int ICON_SIZE = 16;
+		GdlIcons* icons = gdl_icons_new(ICON_SIZE);
+		GdkPixbuf *pixbuf = gdl_icons_get_uri_icon(icons, uri);
+		GtkWidget* image = gtk_image_new_from_pixbuf (pixbuf);
+		gtk_box_pack_start (GTK_BOX(event_hbox), image, FALSE, FALSE, 0);
+		page->mime_icon = image;
+		g_object_unref(pixbuf);
+		g_object_unref(icons);
+	}
+	g_free(uri);
+	
 	gtk_box_pack_start (GTK_BOX(event_hbox), label, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (event_hbox), button15, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(event_hbox), close_pixmap, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (event_hbox), close_button, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (event_box), event_hbox);
 	
 	/* setup the data hierarchy */
@@ -567,12 +626,18 @@ editor_tab_widget_new(AnjutaDocmanPage* page, AnjutaDocman* docman)
 	/* show the widgets of the tab */
 	gtk_widget_show_all(box);
 
-	gtk_signal_connect (GTK_OBJECT (button15), "clicked",
+	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
 				GTK_SIGNAL_FUNC(on_text_editor_notebook_close_page),
 				docman);
+	gtk_signal_connect (GTK_OBJECT (close_button), "enter",
+				GTK_SIGNAL_FUNC(on_text_editor_notebook_close_button_enter_cb),
+				page);
+	gtk_signal_connect (GTK_OBJECT (close_button), "leave",
+				GTK_SIGNAL_FUNC(on_text_editor_notebook_close_button_leave_cb),
+				page);
 
 	page->close_image = close_pixmap;
-	page->close_button = button15;
+	page->close_button = close_button;
 	page->label = label;
 	page->menu_label = menu_label;
 
@@ -608,7 +673,7 @@ on_open_filesel_response (GtkDialog* dialog, gint id, AnjutaDocman *docman)
 
 	if (id != GTK_RESPONSE_ACCEPT)
 	{
-		gtk_widget_hide (docman->priv->save_as_fileselection);
+		gtk_widget_hide (docman->priv->fileselection);
 		return;
 	}
 	
@@ -630,69 +695,6 @@ on_open_filesel_response (GtkDialog* dialog, gint id, AnjutaDocman *docman)
 	}
 }
 
-static void
-save_as_real (AnjutaDocman *docman)
-{
-	IAnjutaEditor *te;
-	const gchar* uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER(docman->priv->save_as_fileselection));
-	if (!uri)
-		return;
-
-	te = anjuta_docman_get_current_editor (docman);
-	g_return_if_fail (te != NULL);
-
-	gtk_widget_hide (docman->priv->save_as_fileselection);
-	
-	/* TODO: Error checking and maybe rehighlite */
-	ianjuta_file_savable_save_as(IANJUTA_FILE_SAVABLE(te), uri, NULL);
-}
-
-static void
-on_save_as_filesel_response (GtkDialog* dialog, gint id, AnjutaDocman *docman)
-{
-	gchar* uri;
-	GnomeVFSURI* vfs_uri;
-	
-	if (id != GTK_RESPONSE_ACCEPT)
-	{
-		gtk_widget_hide (docman->priv->save_as_fileselection);
-		return;
-	} 	
-
-	uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
-	vfs_uri = gnome_vfs_uri_new(uri);
-	if (gnome_vfs_uri_exists(vfs_uri))
-	{
-		GtkWidget *dialog;
-		GtkWidget *parent;
-		parent = gtk_widget_get_toplevel (GTK_WIDGET (docman));
-		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-										 GTK_DIALOG_DESTROY_WITH_PARENT,
-										 GTK_MESSAGE_QUESTION,
-										 GTK_BUTTONS_NONE,
-										 _("The file '%s' already exists.\n"
-										 "Do you want to replace it with the one you are saving?"),
-										 uri);
-		gtk_dialog_add_button (GTK_DIALOG(dialog),
-							   GTK_STOCK_CANCEL,
-							   GTK_RESPONSE_CANCEL);
-		anjuta_util_dialog_add_button (GTK_DIALOG (dialog),
-								  _("_Replace"),
-								  GTK_STOCK_REFRESH,
-								  GTK_RESPONSE_YES);
-		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
-			save_as_real (docman);
-		gtk_widget_destroy (dialog);
-	}
-	else
-		save_as_real (docman);
-	g_free (uri);
-
-	if (anjuta_preferences_get_int (ANJUTA_PREFERENCES (docman->priv->preferences),
-									EDITOR_TABS_ORDERING))
-		anjuta_docman_order_tabs (docman);
-}
-
 static GtkWidget*
 create_file_open_dialog_gui (GtkWindow* parent, AnjutaDocman* docman)
 {
@@ -723,10 +725,6 @@ create_file_save_dialog_gui (GtkWindow* parent, AnjutaDocman* docman)
 									GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 									NULL); 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-	g_signal_connect (G_OBJECT(dialog), "response", 
-					  G_CALLBACK(on_save_as_filesel_response), docman);	
-	g_signal_connect_swapped (G_OBJECT(dialog), "delete-event",
-							  G_CALLBACK(gtk_widget_hide), dialog);	
 	return dialog;
 }
 
@@ -743,35 +741,100 @@ anjuta_docman_open_file (AnjutaDocman *docman)
 	gtk_widget_show (docman->priv->fileselection);
 }
 
-void
-anjuta_docman_save_as_file (AnjutaDocman *docman)
+gboolean
+anjuta_docman_save_editor_as (AnjutaDocman *docman, IAnjutaEditor *te,
+							  GtkWidget *parent_window)
 {
-	IAnjutaEditor *te;
-	const gchar* uri;
+	gchar* uri;
+	GnomeVFSURI* vfs_uri;
+	const gchar* file_uri;
 	const gchar* filename;
+	GtkWidget *parent;
+	GtkWidget *dialog;
+	gint response;
+	gboolean file_saved = TRUE;
 	
-	if (!docman->priv->save_as_fileselection)
-	{
-		GtkWidget *parent;
-		parent = gtk_widget_get_toplevel (GTK_WIDGET (docman));
-		docman->priv->save_as_fileselection =
-			create_file_save_dialog_gui (GTK_WINDOW (parent), docman);
-		gtk_window_set_modal (GTK_WINDOW (docman->priv->save_as_fileselection),
-							  TRUE);
-	}
-	te = anjuta_docman_get_current_editor (docman);
-	if (te == NULL)
-		return;
-	if ((uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL)) != NULL)
-		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER(docman->priv->save_as_fileselection),
-									uri);
-	else if ((filename = ianjuta_editor_get_filename(te, NULL)) != NULL)
-		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(docman->priv->save_as_fileselection),
-											filename);
+	g_return_val_if_fail (ANJUTA_IS_DOCMAN (docman), FALSE);
+	g_return_val_if_fail (IANJUTA_IS_EDITOR (te), FALSE);
+	
+	if (parent_window)
+		parent = parent_window;
 	else
-		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(docman->priv->save_as_fileselection),
-										  "");
-	gtk_widget_show (docman->priv->save_as_fileselection);
+		parent = gtk_widget_get_toplevel (GTK_WIDGET (docman));
+	
+	dialog = create_file_save_dialog_gui (GTK_WINDOW (parent), docman);
+	
+	if ((file_uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL)) != NULL)
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER(dialog), file_uri);
+	else if ((filename = ianjuta_editor_get_filename(te, NULL)) != NULL)
+		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog), filename);
+	else
+		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog), "");
+	
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (response != GTK_RESPONSE_ACCEPT)
+	{
+		gtk_widget_destroy (dialog);
+		return FALSE;
+	}
+	
+	uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
+	vfs_uri = gnome_vfs_uri_new(uri);
+	if (gnome_vfs_uri_exists(vfs_uri))
+	{
+		GtkWidget *msg_dialog;
+		msg_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+											 GTK_DIALOG_DESTROY_WITH_PARENT,
+											 GTK_MESSAGE_QUESTION,
+											 GTK_BUTTONS_NONE,
+											 _("The file '%s' already exists.\n"
+											 "Do you want to replace it with the"
+											 " one you are saving?"),
+											 uri);
+		gtk_dialog_add_button (GTK_DIALOG (msg_dialog),
+							   GTK_STOCK_CANCEL,
+							   GTK_RESPONSE_CANCEL);
+		anjuta_util_dialog_add_button (GTK_DIALOG (msg_dialog),
+								  _("_Replace"),
+								  GTK_STOCK_REFRESH,
+								  GTK_RESPONSE_YES);
+		if (gtk_dialog_run (GTK_DIALOG (msg_dialog)) == GTK_RESPONSE_YES)
+			ianjuta_file_savable_save_as (IANJUTA_FILE_SAVABLE (te), uri,
+										  NULL);
+		else
+			file_saved = FALSE;
+		gtk_widget_destroy (msg_dialog);
+	}
+	else
+	{
+		ianjuta_file_savable_save_as (IANJUTA_FILE_SAVABLE (te), uri, NULL);
+	}
+	
+	if (anjuta_preferences_get_int (ANJUTA_PREFERENCES (docman->priv->preferences),
+									EDITOR_TABS_ORDERING))
+		anjuta_docman_order_tabs (docman);
+
+	gtk_widget_destroy (dialog);
+	g_free (uri);
+	gnome_vfs_uri_unref (vfs_uri);
+	return file_saved;
+}
+
+gboolean
+anjuta_docman_save_editor (AnjutaDocman *docman, IAnjutaEditor *te,
+						   GtkWidget *parent_window)
+{
+	if (ianjuta_file_get_uri(IANJUTA_FILE(te), NULL) == NULL)
+	{
+		anjuta_docman_set_current_editor (docman, te);
+		return anjuta_docman_save_editor_as (docman, te, parent_window);
+	}
+	else
+	{
+		/* TODO: Error checking */
+		ianjuta_file_savable_save(IANJUTA_FILE_SAVABLE(te), NULL);
+	}
+	return TRUE;
 }
 
 static void
@@ -822,8 +885,6 @@ anjuta_docman_finalize (GObject *obj)
 	{
 		if (docman->priv->fileselection)
 			gtk_widget_destroy (docman->priv->fileselection);
-		if (docman->priv->save_as_fileselection)
-			gtk_widget_destroy (docman->priv->save_as_fileselection);
 		g_free (docman->priv);
 		docman->priv = NULL;
 	}
@@ -836,7 +897,6 @@ anjuta_docman_instance_init (AnjutaDocman *docman)
 	docman->priv = g_new0 (AnjutaDocmanPriv, 1);
 	docman->priv->popup_menu = NULL;
 	docman->priv->fileselection = NULL;
-	docman->priv->save_as_fileselection = NULL;
 	
 	gtk_notebook_popup_enable (GTK_NOTEBOOK (docman));
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (docman), TRUE);
@@ -902,13 +962,16 @@ anjuta_docman_class_init (AnjutaDocmanClass *klass)
 }
 
 GtkWidget*
-anjuta_docman_new (AnjutaPreferences *pref)
+anjuta_docman_new (DocmanPlugin* plugin, AnjutaPreferences *pref)
 {
 
 	GtkWidget *docman = NULL;
 	docman = gtk_widget_new (ANJUTA_TYPE_DOCMAN, NULL);
 	if (docman)
-	    ANJUTA_DOCMAN (docman)->priv->preferences = pref;
+	{
+		ANJUTA_DOCMAN (docman)->priv->plugin = plugin;
+		ANJUTA_DOCMAN (docman)->priv->preferences = pref;
+	}
 
 	return docman;
 }
@@ -1074,7 +1137,7 @@ anjuta_docman_add_editor (AnjutaDocman *docman, const gchar *uri,
 
 	docman->priv->editors = g_list_append (docman->priv->editors, (gpointer)page);
 
-	ruri =  gnome_vfs_format_uri_for_display (uri);
+	ruri =  gnome_vfs_format_uri_for_display (uri ? uri : "");
 	
 	/* set the tooltips */	
 	tip = g_markup_printf_escaped("<b>%s</b> %s\n",
@@ -1117,6 +1180,7 @@ anjuta_docman_add_editor (AnjutaDocman *docman, const gchar *uri,
 				   
 	g_signal_emit_by_name (docman, "editor-added", te);
 	anjuta_docman_set_current_editor(docman, te);
+	anjuta_shell_present_widget( ANJUTA_SHELL (docman->shell), GTK_WIDGET (docman), NULL);
 	return te;
 }
 
@@ -1161,8 +1225,10 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 		AnjutaDocmanPage *page = anjuta_docman_page_from_widget (docman, ote);
 		if (page && page->close_button != NULL)
 		{
-			gtk_widget_hide (page->close_button);
-			gtk_widget_show (page->close_image);
+			gtk_widget_set_sensitive (page->close_image, FALSE);
+			if (page->mime_icon)
+				gtk_widget_set_sensitive (page->mime_icon, FALSE);
+			page->is_current = FALSE;
 		}
 	}
 	docman->priv->current_editor = te;
@@ -1172,8 +1238,10 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 		AnjutaDocmanPage *page = anjuta_docman_page_from_widget (docman, te);
 		if (page && page->close_button != NULL)
 		{
-			gtk_widget_show (page->close_button);
-			gtk_widget_hide (page->close_image);
+			gtk_widget_set_sensitive (page->close_image, TRUE);
+			if (page->mime_icon)
+				gtk_widget_set_sensitive (page->mime_icon, TRUE);
+			page->is_current = TRUE;
 		}
 		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (docman),
 										  GTK_WIDGET (te));
@@ -1226,7 +1294,7 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 IAnjutaEditor *
 anjuta_docman_goto_file_line (AnjutaDocman *docman, const gchar *fname, glong lineno)
 {
-	return anjuta_docman_goto_file_line_mark (docman, fname, lineno, TRUE);
+	return anjuta_docman_goto_file_line_mark (docman, fname, lineno, FALSE);
 }
 
 IAnjutaEditor *
@@ -1239,7 +1307,9 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 	GList *node;
 	const gchar *linenum;
 	glong lineno;
-
+	gboolean is_local_uri;
+	gchar *normalized_path = NULL;
+	
 	IAnjutaEditor *te;
 
 	g_return_val_if_fail (fname, NULL);
@@ -1258,6 +1328,14 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 	/* Restore URI without fragement identifier (linenum) */
 	uri = gnome_vfs_uri_to_string (vfs_uri,
 								   GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
+	
+	/* Get the normalized file path for comparision */
+	is_local_uri = gnome_vfs_uri_is_local (vfs_uri);
+	if (is_local_uri)
+		normalized_path = realpath (gnome_vfs_uri_get_path (vfs_uri), NULL);
+	if (normalized_path == NULL)
+		normalized_path = g_strdup (uri);
+	
 	gnome_vfs_uri_unref (vfs_uri);
 	/* g_free(filename); */
 
@@ -1268,15 +1346,30 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 	while (node)
 	{
 		AnjutaDocmanPage *page;
+		gboolean te_is_local_uri;
+		gchar *te_normalized_path = NULL;
+		
 		page = (AnjutaDocmanPage *) node->data;
 		te = IANJUTA_EDITOR (page->widget);
 		te_uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
+		
 		if (te_uri == NULL)
 		{
 			node = g_list_next (node);
 			continue;
 		}
-		if (strcmp (uri, te_uri) == 0)
+		
+		/* Get the normalized file path for comparision */
+		vfs_uri = gnome_vfs_uri_new (te_uri);
+		te_is_local_uri = gnome_vfs_uri_is_local (vfs_uri);
+		if (te_is_local_uri)
+			te_normalized_path = realpath (gnome_vfs_uri_get_path (vfs_uri),
+										   NULL);
+		if (te_normalized_path == NULL)
+			te_normalized_path = g_strdup (te_uri);
+		gnome_vfs_uri_unref (vfs_uri);
+		
+		if (strcmp (normalized_path, te_normalized_path) == 0)
 		{
 			if (lineno >= 0)
 			{
@@ -1289,12 +1382,16 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 					ianjuta_markable_mark(IANJUTA_MARKABLE(te), lineno,
   	                				IANJUTA_MARKABLE_BASIC, NULL);
 				}
-  	        }				
+  	        }
 			anjuta_docman_show_editor (docman, GTK_WIDGET (te));
 			g_free (uri);
-			an_file_history_push (ianjuta_file_get_uri(IANJUTA_FILE(te), NULL), lineno);
+			g_free (normalized_path);
+			g_free (te_normalized_path);
+			an_file_history_push (ianjuta_file_get_uri(IANJUTA_FILE(te),
+													   NULL), lineno);
 			return te;
 		}
+		g_free (te_normalized_path);
 		node = g_list_next (node);
 	}
 	te = anjuta_docman_add_editor (docman, uri, NULL);
@@ -1313,6 +1410,7 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 		}
 	}
 	g_free (uri);
+	g_free (normalized_path);
 	return te ;
 }
 
@@ -1400,6 +1498,7 @@ anjuta_docman_show_editor (AnjutaDocman *docman, GtkWidget* te)
 		{
 			gtk_notebook_set_current_page (GTK_NOTEBOOK (docman), i);
 			anjuta_docman_set_current_editor (docman, IANJUTA_EDITOR (te));
+			anjuta_shell_present_widget (ANJUTA_SHELL (docman->shell), GTK_WIDGET (docman), NULL);
 			return;
 		}
 		i++;
@@ -1519,7 +1618,7 @@ anjuta_docman_get_editor_from_path (AnjutaDocman *docman, const gchar *szFullPat
 	while (node)
 	{
 		page = node->data;
-		te = (IAnjutaEditor *) page->widget;
+		te = IANJUTA_EDITOR (page->widget);
 		uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
 		if (uri != NULL)
 		{
@@ -1681,7 +1780,7 @@ anjuta_docman_get_all_editors (AnjutaDocman *docman)
 		AnjutaDocmanPage *page;
 		IAnjutaEditor *te;
 		page = (AnjutaDocmanPage *) node->data;
-		te = (IAnjutaEditor *) page->widget;
+		te = IANJUTA_EDITOR (page->widget);
 		editors = g_list_prepend (editors, te);
 		node = g_list_next (node);
 	}
@@ -1700,7 +1799,7 @@ anjuta_docman_set_busy (AnjutaDocman *docman, gboolean state)
 		AnjutaDocmanPage *page;
 		IAnjutaEditor *te;
 		page = (AnjutaDocmanPage *) node->data;
-		te = (IAnjutaEditor *) page->widget;
+		te = IANJUTA_EDITOR (page->widget);
 		/* FIXME:
 		text_editor_set_busy (te, state);*/
 		node = g_list_next (node);
