@@ -23,6 +23,8 @@
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/anjuta-encodings.h>
+#include <libanjuta/plugins.h>
+
 #include <libegg/menu/egg-entry-action.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
@@ -38,7 +40,9 @@
 #include <libanjuta/interfaces/ianjuta-editor-zoom.h>
 #include <libanjuta/interfaces/ianjuta-editor-goto.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
-
+#include <libanjuta/interfaces/ianjuta-editor-language.h>
+#include <libanjuta/interfaces/ianjuta-language-support.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 
 #include "anjuta-docman.h"
 #include "action-callbacks.h"
@@ -145,22 +149,22 @@ static ShortcutMapping global_keymap[] = {
 
 static GtkActionEntry actions_file[] = {
   { "ActionFileSave", N_("_Save"), GTK_STOCK_SAVE, "<control>s",
-	N_("Save current file"), G_CALLBACK (on_save1_activate)},
+	N_("Save current file"), G_CALLBACK (on_save_activate)},
   { "ActionFileSaveAs", N_("Save _As..."), GTK_STOCK_SAVE_AS, NULL,
 	N_("Save the current file with a different name"),
-    G_CALLBACK (on_save_as1_activate)},
+    G_CALLBACK (on_save_as_activate)},
   { "ActionFileSaveAll", N_("Save A_ll"),  NULL, NULL,
 	N_("Save all currently open files, except new files"),
-    G_CALLBACK (on_save_all1_activate)},
+    G_CALLBACK (on_save_all_activate)},
   { "ActionFileClose", N_("_Close File"), GTK_STOCK_CLOSE, "<control>w",
 	N_("Close current file"),
-    G_CALLBACK (on_close_file1_activate)},
-  { "ActionFileCloseAll", N_("Close All Files"), GTK_STOCK_CLOSE, "<alt>d",
+    G_CALLBACK (on_close_file_activate)},
+  { "ActionFileCloseAll", N_("Close All Files"), GTK_STOCK_CLOSE, "<shift><control>w",
 	N_("Close all files"),
-    G_CALLBACK (on_close_all_file1_activate)},
+    G_CALLBACK (on_close_all_file_activate)},
   { "ActionFileReload", N_("Reload F_ile"), GTK_STOCK_REVERT_TO_SAVED, NULL,
 	N_("Reload current file"),
-    G_CALLBACK (on_reload_file1_activate)},
+    G_CALLBACK (on_reload_file_activate)},
   { "ActionFileSwap", N_("Swap .h/.c"), ANJUTA_STOCK_SWAP, NULL,
 	N_("Swap c header and source file"),
     G_CALLBACK (on_swap_activate)},
@@ -168,7 +172,7 @@ static GtkActionEntry actions_file[] = {
 };
 
 static GtkActionEntry actions_print[] = {
-  { "ActionPrintFile", N_("_Print"), GTK_STOCK_PRINT, "<alt>p",
+  { "ActionPrintFile", N_("_Print..."), GTK_STOCK_PRINT, "<control>p",
 	N_("Print the current file"), G_CALLBACK (anjuta_print_cb)},
   { "ActionPrintPreview", N_("_Print Preview"), NULL, NULL,
 	N_("Print preview of the current file"),
@@ -183,16 +187,16 @@ static GtkActionEntry actions_transform[] = {
   { "ActionEditMakeSelectionLowercase", N_("Make Selection Lowercase"), NULL, NULL,
 	N_("Make the selected text lowercase"),
     G_CALLBACK (on_editor_command_lower_case_activate)},
-  { "ActionEditConvertCRLF", N_("Convert EOL chars to CRLF"), NULL, NULL,
+  { "ActionEditConvertCRLF", N_("Convert EOL to CRLF"), NULL, NULL,
 	N_("Convert End Of Line characters to DOS EOL (CRLF)"),
     G_CALLBACK (on_editor_command_eol_crlf_activate)},
-  { "ActionEditConvertLF", N_("Convert EOL chars to LF"), NULL, NULL,
+  { "ActionEditConvertLF", N_("Convert EOL to LF"), NULL, NULL,
 	N_("Convert End Of Line characters to Unix EOL (LF)"),
     G_CALLBACK (on_editor_command_eol_lf_activate)},
-  { "ActionEditConvertCR", N_("Convert EOL chars to CR"), NULL, NULL,
+  { "ActionEditConvertCR", N_("Convert EOL to CR"), NULL, NULL,
 	N_("Convert End Of Line characters to Mac OS EOL (CR)"),
     G_CALLBACK (on_editor_command_eol_cr_activate)},
-  { "ActionEditConvertEOL", N_("Convert EOL chars to majority EOL"), NULL, NULL,
+  { "ActionEditConvertEOL", N_("Convert EOL to Majority EOL"), NULL, NULL,
 	N_("Convert End Of Line characters to majority of the EOL found in the file"),
     G_CALLBACK (on_transform_eolchars1_activate)},
 };
@@ -202,17 +206,17 @@ static GtkActionEntry actions_select[] = {
   { "ActionEditSelectAll", N_("Select _All"), NULL, "<control>a",
 	N_("Select all text in the editor"),
     G_CALLBACK (on_editor_command_select_all_activate)},
-  { "ActionEditSelectToBrace", N_("Select to _Brace"), NULL, "<alt>a",
+  { "ActionEditSelectToBrace", N_("Select to _Brace"), NULL, NULL,
 	N_("Select the text in the matching braces"),
     G_CALLBACK (on_editor_command_select_to_brace_activate)},
   { "ActionEditSelectBlock", N_("Select _Code Block"),
-	ANJUTA_STOCK_BLOCK_SELECT, "<alt>b",
+	ANJUTA_STOCK_BLOCK_SELECT, "<shift><control>b",
 	N_("Select the current code block"),
     G_CALLBACK (on_editor_command_select_block_activate)},
 };
 
 static GtkActionEntry actions_comment[] = {
-  { "ActionMenuEditComment", N_("Co_mment code"), NULL, NULL, NULL, NULL},
+  { "ActionMenuEditComment", N_("Co_mment"), NULL, NULL, NULL, NULL},
   { "ActionEditCommentBlock", N_("_Block Comment/Uncomment"), NULL, NULL,
 	N_("Block comment the selected text"),
     G_CALLBACK (on_comment_block)},
@@ -230,7 +234,7 @@ static GtkActionEntry actions_navigation[] = {
 	GTK_STOCK_JUMP_TO, NULL,
 	N_("Go to a particular line in the editor"),
     G_CALLBACK (on_goto_activate)},
-  { "ActionEditGotoLine", N_("_Line number..."),
+  { "ActionEditGotoLine", N_("_Line Number..."),
 	GTK_STOCK_JUMP_TO, "<control><alt>g",
 	N_("Go to a particular line in the editor"),
     G_CALLBACK (on_goto_line_no1_activate)},
@@ -238,19 +242,19 @@ static GtkActionEntry actions_navigation[] = {
 	GTK_STOCK_JUMP_TO, "<control><alt>m", 
 	N_("Go to the matching brace in the editor"),
     G_CALLBACK (on_editor_command_match_brace_activate)},
-  { "ActionEditGotoBlockStart", N_("_Start of block"),
+  { "ActionEditGotoBlockStart", N_("_Start of Block"),
 	ANJUTA_STOCK_BLOCK_START, "<control><alt>s",
 	N_("Go to the start of the current block"),
     G_CALLBACK (on_goto_block_start1_activate)},
-  { "ActionEditGotoBlockEnd", N_("_End of block"),
+  { "ActionEditGotoBlockEnd", N_("_End of Block"),
 	ANJUTA_STOCK_BLOCK_END, "<control><alt>e",
 	N_("Go to the end of the current block"),
     G_CALLBACK (on_goto_block_end1_activate)},
-  { "ActionEditGotoHistoryPrev", N_("Previous _history"),
+  { "ActionEditGotoHistoryPrev", N_("Previous _History"),
 	GTK_STOCK_JUMP_TO, NULL,
 	N_("Goto previous history"),
     G_CALLBACK (on_prev_history)},
-  { "ActionEditGotoHistoryNext", N_("Next histor_y"),
+  { "ActionEditGotoHistoryNext", N_("Next Histor_y"),
 	GTK_STOCK_JUMP_TO, NULL,
 	N_("Goto next history"),
     G_CALLBACK (on_next_history)}
@@ -290,34 +294,34 @@ static GtkActionEntry actions_edit[] = {
 };
 
 static GtkToggleActionEntry actions_view[] = {
-  { "ActionViewEditorLinenumbers", N_("_Line numbers margin"), NULL, NULL,
+  { "ActionViewEditorLinenumbers", N_("_Line Number Margin"), NULL, NULL,
 	N_("Show/Hide line numbers"),
     G_CALLBACK (on_editor_linenos1_activate), FALSE},
-  { "ActionViewEditorMarkers", N_("_Markers Margin"), NULL, NULL,
-	N_("Show/Hide markers margin"),
+  { "ActionViewEditorMarkers", N_("_Marker Margin"), NULL, NULL,
+	N_("Show/Hide marker margin"),
     G_CALLBACK (on_editor_markers1_activate), FALSE},
-  { "ActionViewEditorFolds", N_("_Code fold margin"), NULL, NULL,
+  { "ActionViewEditorFolds", N_("_Code Fold Margin"), NULL, NULL,
 	N_("Show/Hide code fold margin"),
     G_CALLBACK (on_editor_codefold1_activate), FALSE},
-  { "ActionViewEditorGuides", N_("_Indentation guides"), NULL, NULL,
+  { "ActionViewEditorGuides", N_("_Indentation Guides"), NULL, NULL,
 	N_("Show/Hide indentation guides"),
     G_CALLBACK (on_editor_indentguides1_activate), FALSE},
-  { "ActionViewEditorSpaces", N_("_White spaces"), NULL, NULL,
+  { "ActionViewEditorSpaces", N_("_White Space"), NULL, NULL,
 	N_("Show/Hide white spaces"),
     G_CALLBACK (on_editor_whitespaces1_activate), FALSE},
-  { "ActionViewEditorEOL", N_("_Line end characters"), NULL, NULL,
+  { "ActionViewEditorEOL", N_("_Line End Characters"), NULL, NULL,
 	N_("Show/Hide line end characters"),
     G_CALLBACK (on_editor_eolchars1_activate), FALSE},
-  { "ActionViewEditorWrapping", N_("Line _wrapping"), NULL, NULL,
+  { "ActionViewEditorWrapping", N_("Line _Wrapping"), NULL, NULL,
 	N_("Enable/disable line wrapping"),
     G_CALLBACK (on_editor_linewrap1_activate), FALSE}
 };
 
 static GtkActionEntry actions_zoom[] = {
-  { "ActionViewEditorZoomIn", N_("Zoom in"), NULL, NULL,
+  { "ActionViewEditorZoomIn", N_("Zoom In"), GTK_STOCK_ZOOM_IN, "<control>plus",
 	N_("Zoom in: Increase font size"),
     G_CALLBACK (on_zoom_in_text_activate)},
-  { "ActionViewEditorZoomOut", N_("Zoom out"), NULL, NULL,
+  { "ActionViewEditorZoomOut", N_("Zoom Out"), GTK_STOCK_ZOOM_OUT, "<control>minus",
 	N_("Zoom out: Decrease font size"),
     G_CALLBACK (on_zoom_out_text_activate)}
 };
@@ -362,27 +366,27 @@ static GtkActionEntry actions_format[] = {
 
 static GtkActionEntry actions_bookmark[] = {
   { "ActionMenuBookmark", N_("Bookmar_k"), NULL, NULL, NULL, NULL},
-  { "ActionBookmarkToggle", N_("_Toggle bookmark"),
+  { "ActionBookmarkToggle", N_("_Toggle Bookmark"),
 	ANJUTA_STOCK_BOOKMARK_TOGGLE, "<control>k",
 	N_("Toggle a bookmark at the current line position"),
     G_CALLBACK (on_editor_command_bookmark_toggle_activate)},
-  { "ActionBookmarkFirst", N_("_First bookmark"),
+  { "ActionBookmarkFirst", N_("_First Bookmark"),
 	ANJUTA_STOCK_BOOKMARK_FIRST, NULL,
 	N_("Jump to the first bookmark in the file"),
     G_CALLBACK (on_editor_command_bookmark_first_activate)},
-  { "ActionBookmarkPrevious", N_("_Previous bookmark"),
+  { "ActionBookmarkPrevious", N_("_Previous Bookmark"),
 	ANJUTA_STOCK_BOOKMARK_PREV, "<control>comma",
 	N_("Jump to the previous bookmark in the file"),
     G_CALLBACK (on_editor_command_bookmark_prev_activate)},
-  { "ActionBookmarkNext", N_("_Next bookmark"),
+  { "ActionBookmarkNext", N_("_Next Bookmark"),
 	ANJUTA_STOCK_BOOKMARK_NEXT, "<control>period",
 	N_("Jump to the next bookmark in the file"),
     G_CALLBACK (on_editor_command_bookmark_next_activate)},
-  { "ActionBookmarkLast", N_("_Last bookmark"),
+  { "ActionBookmarkLast", N_("_Last Bookmark"),
 	ANJUTA_STOCK_BOOKMARK_LAST, NULL,
 	N_("Jump to the last bookmark in the file"),
     G_CALLBACK (on_editor_command_bookmark_last_activate)},
-  { "ActionBookmarkClear", N_("_Clear all bookmarks"),
+  { "ActionBookmarkClear", N_("_Clear All Bookmarks"),
 	NULL, NULL,
 	N_("Clear bookmarks"),
     G_CALLBACK (on_editor_command_bookmark_clear_activate)},
@@ -476,7 +480,7 @@ ui_states_init (AnjutaPlugin *plugin)
 		VIEW_LINE_WRAP
 	};
 	
-	eplugin = (DocmanPlugin*)plugin;
+	eplugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	for (i = 0; i < sizeof (actions_view)/sizeof(GtkToggleActionEntry); i++)
 	{
 		GtkAction *action;
@@ -596,6 +600,13 @@ update_editor_ui_interface_items (AnjutaPlugin *plugin, IAnjutaEditor *editor)
 	
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 
+	/* IAnjutaEditorLanguage */
+	flag = IANJUTA_IS_EDITOR_LANGUAGE (editor);
+	
+	action = anjuta_ui_get_action (ui, "ActionGroupEditorStyle",
+								   "ActionMenuFormatStyle");
+	g_object_set (G_OBJECT (action), "visible", flag, NULL);
+	
 	/* IAnjutaEditorSelection */
 	flag = IANJUTA_IS_EDITOR_SELECTION (editor);
 	
@@ -850,10 +861,110 @@ on_editor_update_ui (IAnjutaEditor *editor, DocmanPlugin *plugin)
 		update_status (plugin, te);
 }
 
+/* Remove all instances of c from the string s. */
+static void remove_char(gchar *s, gchar c)
+{
+	gchar *t = s;
+	for (; *s ; ++s)
+		if (*s != c)
+			*t++ = *s;
+	*t = '\0';
+}
+
+/* Compare two strings, ignoring _ characters which indicate mneumonics.
+ * Returns -1, 0, or 1, just like strcmp(). */
+static gint
+menu_name_compare(const gchar *s, const char *t)
+{
+	gchar *s1 = g_utf8_strdown(s, -1);
+	gchar *t1 = g_utf8_strdown(t, -1);
+	remove_char(s1, '_');
+	remove_char(t1, '_');
+	int result = g_utf8_collate(s1, t1);
+	g_free(s1);
+	g_free(t1);
+	return result;
+}
+
+static gint
+compare_language_name(const gchar *lang1, const gchar *lang2, IAnjutaEditorLanguage *manager)
+{
+	const gchar *fullname1 = ianjuta_editor_language_get_language_name (manager, lang1, NULL);
+	const gchar *fullname2 = ianjuta_editor_language_get_language_name (manager, lang2, NULL);
+	return menu_name_compare(fullname1, fullname2);
+}
+
+static GtkWidget*
+create_highlight_submenu (DocmanPlugin *plugin, IAnjutaEditor *editor)
+{
+	const GList *languages, *node;
+	GList *sorted_languages;
+	GtkWidget *submenu;
+	GtkWidget *menuitem;
+	
+	submenu = gtk_menu_new ();
+	
+	if (!editor || !IANJUTA_IS_EDITOR_LANGUAGE (editor))
+		return NULL;
+	
+	languages = ianjuta_editor_language_get_supported_languages (IANJUTA_EDITOR_LANGUAGE (editor), NULL);
+	if (!languages)
+		return NULL;
+	
+	/* Automatic highlight menu */
+	menuitem = gtk_menu_item_new_with_mnemonic (_("Automatic"));
+	g_signal_connect (G_OBJECT (menuitem), "activate",
+					  G_CALLBACK (on_force_hilite_activate),
+					  plugin);
+	g_object_set_data(G_OBJECT(menuitem), "language_code", "auto-detect");
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), gtk_separator_menu_item_new());
+
+	/* Sort languages so they appear alphabetically in the menu. */
+	sorted_languages = g_list_copy((GList *) languages);
+	sorted_languages = g_list_sort_with_data(sorted_languages, (GCompareDataFunc) compare_language_name,
+											 IANJUTA_EDITOR_LANGUAGE (editor));
+	
+	node = sorted_languages;
+	while (node)
+	{
+		const gchar *lang = node->data;
+		const gchar *name = ianjuta_editor_language_get_language_name (IANJUTA_EDITOR_LANGUAGE (editor), lang, NULL);
+		
+		menuitem = gtk_menu_item_new_with_mnemonic (name);
+		g_object_set_data_full (G_OBJECT (menuitem), "language_code",
+								g_strdup (lang),
+								(GDestroyNotify)g_free);
+		g_signal_connect (G_OBJECT (menuitem), "activate",
+						  G_CALLBACK (on_force_hilite_activate),
+						  plugin);
+		gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+		node = g_list_next (node);
+	}
+	g_list_free(sorted_languages);
+	
+	gtk_widget_show_all (submenu);
+	return submenu;
+}
+
 static void
 on_editor_added (AnjutaDocman *docman, IAnjutaEditor *te,
 				   AnjutaPlugin *plugin)
 {
+	GtkWidget *highlight_submenu, *highlight_menu;
+	DocmanPlugin *editor_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
+	
+	/* Create Highlight submenu */
+	highlight_submenu = 
+		create_highlight_submenu (editor_plugin, te);
+	if (highlight_submenu)
+	{
+		highlight_menu =
+			gtk_ui_manager_get_widget (GTK_UI_MANAGER (editor_plugin->ui),
+						"/MenuMain/MenuView/MenuViewEditor/MenuFormatStyle");
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (highlight_menu),
+								   highlight_submenu);
+	}
 	g_signal_connect (G_OBJECT (te), "update-ui",
 					  G_CALLBACK (on_editor_update_ui),
 					  ANJUTA_PLUGIN (plugin));
@@ -868,10 +979,24 @@ static void
 on_editor_changed (AnjutaDocman *docman, IAnjutaEditor *te,
 				   AnjutaPlugin *plugin)
 {
-	update_status ((DocmanPlugin *)plugin, te);
+	DocmanPlugin *docman_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
+	
+	update_status (docman_plugin, te);
 	update_editor_ui (plugin, te);
+		
+	/* unload previous support plugins */
+	if (docman_plugin->support_plugins) {
+		g_list_foreach (docman_plugin->support_plugins,
+						(GFunc) anjuta_plugin_deactivate, NULL);
+		g_list_free (docman_plugin->support_plugins);
+		docman_plugin->support_plugins = NULL;
+	}
+	
 	if (te)
 	{
+		const gchar *language;
+		GSList *support_plugin_descs, *node;
+		
 		GValue *value = g_new0 (GValue, 1);
 		g_value_init (value, G_TYPE_OBJECT);
 		g_value_set_object (value, te);
@@ -879,6 +1004,42 @@ on_editor_changed (AnjutaDocman *docman, IAnjutaEditor *te,
 								"document_manager_current_editor",
 								value, NULL);
 		DEBUG_PRINT ("Editor Added");
+		
+		/* Load current language editor support plugins */
+		if (IANJUTA_IS_EDITOR_LANGUAGE (te)) {
+			language = ianjuta_editor_language_get_language (IANJUTA_EDITOR_LANGUAGE (te), NULL);
+			support_plugin_descs = anjuta_plugins_query (plugin->shell,
+														 "Anjuta Plugin",
+														 "Interfaces",
+														 "IAnjutaLanguageSupport",
+														 "Language Support",
+														 "Languages",
+														 language, NULL);
+			node = support_plugin_descs;
+			while (node) {
+				gchar *plugin_id;
+				GObject *plugin_object;
+				
+				AnjutaPluginDescription *desc = node->data;
+				anjuta_plugin_description_get_string (desc, "Anjuta Plugin", "Location",
+													  &plugin_id);
+				plugin_object = anjuta_plugins_get_plugin_by_id (plugin->shell,
+																 plugin_id);
+				/* anjuta_plugin_activate (ANJUTA_PLUGIN (plugin_object)); */
+				if (plugin_object)
+				{
+					docman_plugin->support_plugins = g_list_prepend (docman_plugin->support_plugins,
+																	 plugin_object);
+				}
+				else
+				{
+					DEBUG_PRINT ("Failed to load support plugin %s", plugin_id);
+				}
+				node = node->next;
+				g_free (plugin_id);
+			}
+			g_slist_free (support_plugin_descs);
+		}
 	}
 	else
 	{
@@ -1048,6 +1209,43 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 	g_list_free (files);
 }
 
+static gboolean
+on_save_prompt_save_editor (AnjutaSavePrompt *save_prompt,
+							gpointer item, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	return anjuta_docman_save_editor (ANJUTA_DOCMAN (plugin->docman),
+									  IANJUTA_EDITOR (item),
+									  GTK_WIDGET (save_prompt));
+}
+
+static void
+on_save_prompt (AnjutaShell *shell, AnjutaSavePrompt *save_prompt,
+				DocmanPlugin *plugin)
+{
+	GList *list, *node;
+	
+	list = anjuta_docman_get_all_editors (ANJUTA_DOCMAN (plugin->docman));
+	node = list;
+	while (node)
+	{
+		IAnjutaFileSavable *editor = IANJUTA_FILE_SAVABLE (node->data);
+		if (ianjuta_file_savable_is_dirty (editor, NULL))
+		{
+			const gchar *name;
+			const gchar *uri;
+			
+			name = ianjuta_editor_get_filename (IANJUTA_EDITOR (editor), NULL);
+			uri = ianjuta_file_get_uri (IANJUTA_FILE (editor), NULL);
+			anjuta_save_prompt_add_item (save_prompt, name, uri, editor,
+										 on_save_prompt_save_editor, plugin);
+		}
+		node = g_list_next (node);
+	}
+}
+
 static void
 docman_plugin_set_tab_pos (DocmanPlugin *ep)
 {
@@ -1073,9 +1271,9 @@ docman_plugin_set_tab_pos (DocmanPlugin *ep)
 				pos = GTK_POS_RIGHT;
 			else if (strcasecmp (tab_pos, "bottom") == 0)
 				pos = GTK_POS_BOTTOM;
+			g_free (tab_pos);
 		}
 		gtk_notebook_set_tab_pos (GTK_NOTEBOOK (ep->docman), pos);
-		g_free (tab_pos);
 	}
 }
 
@@ -1083,23 +1281,23 @@ static void
 on_gconf_notify_prefs (GConfClient *gclient, guint cnxn_id,
 					   GConfEntry *entry, gpointer user_data)
 {
-	DocmanPlugin *ep = (DocmanPlugin*)user_data;
+	DocmanPlugin *ep = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman_plugin_set_tab_pos (ep);
 }
 
 static gboolean
 on_docman_auto_save (gpointer data)
 {
-	DocmanPlugin *plugin = (DocmanPlugin*)data;
+	DocmanPlugin *plugin = ANJUTA_PLUGIN_DOCMAN (data);
 	AnjutaShell* shell;
 	AnjutaPreferences* prefs;
-    AnjutaDocman *docman;
-    AnjutaStatus* status;
+	AnjutaDocman *docman;
+	AnjutaStatus* status;
 	IAnjutaEditor* editor;
 	GList* editors;
 	docman = ANJUTA_DOCMAN (plugin->docman);
 	
-    g_object_get(G_OBJECT(plugin), "shell", &shell, NULL);
+	g_object_get(G_OBJECT(plugin), "shell", &shell, NULL);
 	prefs = anjuta_shell_get_preferences(shell, NULL);
 	status = anjuta_shell_get_status(shell, NULL);
 	
@@ -1117,11 +1315,11 @@ on_docman_auto_save (gpointer data)
 		editor = IANJUTA_EDITOR(editors->data);
 		if (ianjuta_file_savable_is_dirty(IANJUTA_FILE_SAVABLE(editor), NULL))
 		{
-            if (ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL) != NULL)
-            {
-                ianjuta_file_savable_save(IANJUTA_FILE_SAVABLE(editor), NULL);
-            }
-        }
+			if (ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL) != NULL)
+			{
+				ianjuta_file_savable_save(IANJUTA_FILE_SAVABLE(editor), NULL);
+			}
+		}
 		editors = g_list_next(editors);
 	}
 	// TODO: Check for errors
@@ -1138,7 +1336,7 @@ static void
 on_gconf_notify_timer (GConfClient *gclient, guint cnxn_id,
 					   GConfEntry *entry, gpointer user_data)
 {
-	DocmanPlugin *ep = (DocmanPlugin*)user_data;
+	DocmanPlugin *ep = ANJUTA_PLUGIN_DOCMAN (user_data);
 	AnjutaShell* shell;
 	AnjutaPreferences* prefs;
 	gint auto_save_timer;
@@ -1214,67 +1412,6 @@ prefs_finalize (DocmanPlugin *ep)
 	ep->gconf_notify_ids = NULL;
 }
 
-static GtkWidget*
-create_highlight_submenu (DocmanPlugin *plugin)
-{
-	gchar *menu_entries;
-	GtkWidget *submenu;
-	gchar **strv;
-	gchar **token;
-	GtkWidget *menuitem;
-	
-	submenu = gtk_menu_new ();
-	
-	/* Automatic highlight menu */
-	menuitem = gtk_menu_item_new_with_mnemonic (_("Automatic"));
-	g_signal_connect (G_OBJECT (menuitem), "activate",
-					  G_CALLBACK (on_force_hilite_activate),
-					  plugin);
-	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
-
-	//menu_entries = prop_get (text_editor_get_props (), "menu.language");
-	menu_entries = "";
-	g_return_val_if_fail (menu_entries != NULL, NULL);
-	strv = g_strsplit (menu_entries, "|", -1);
-	token = strv;
-	while (*token)
-	{
-		gchar *iter;
-		gchar *name, *extension;
-		
-		name = *token++;
-		if (!name)
-			break;
-		
-		extension = *token++;
-		if (!extension)
-			break;
-		token++;
-		
-		if (name[0] == '#')
-			continue;
-		
-		iter = name;
-		while (*iter)
-		{
-			if (*iter == '&')
-				*iter = '_';
-			iter++;
-		}
-		menuitem = gtk_menu_item_new_with_mnemonic (name);
-		g_object_set_data_full (G_OBJECT (menuitem), "file_extension",
-								g_strconcat ("file.", extension, NULL),
-								(GDestroyNotify)g_free);
-		g_signal_connect (G_OBJECT (menuitem), "activate",
-						  G_CALLBACK (on_force_hilite_activate),
-						  plugin);
-		gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
-	}
-	g_strfreev (strv);
-	gtk_widget_show_all (submenu);
-	return submenu;
-}
-
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
@@ -1283,22 +1420,21 @@ activate_plugin (AnjutaPlugin *plugin)
 	DocmanPlugin *editor_plugin;
 	GtkActionGroup *group;
 	GtkAction *action;
-	GladeXML *gxml;
 	gint i;
 	AnjutaStatus *status;
-	GtkWidget *highlight_submenu, *highlight_menu;
 	static gboolean initialized = FALSE;
 	
 	DEBUG_PRINT ("DocmanPlugin: Activating Editor plugin...");
 	
-	editor_plugin = (DocmanPlugin*) plugin;
+	editor_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	editor_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	editor_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	status = anjuta_shell_get_status (plugin->shell, NULL);
 	
 	ui = editor_plugin->ui;
-	docman = anjuta_docman_new (editor_plugin->prefs);
+	docman = anjuta_docman_new (editor_plugin, editor_plugin->prefs);
 	editor_plugin->docman = docman;
+	ANJUTA_DOCMAN(docman)->shell = plugin->shell;
 	g_signal_connect (G_OBJECT (docman), "editor-added",
 					  G_CALLBACK (on_editor_added), plugin);
 	g_signal_connect (G_OBJECT (docman), "editor-changed",
@@ -1312,38 +1448,8 @@ activate_plugin (AnjutaPlugin *plugin)
 	
 	if (!initialized)
 	{
-		GtkWidget *indent_button;
-		GtkWidget *indent_combo;
-		GtkWidget *indent_entry;
-		
-		ANJUTA_DOCMAN(docman)->shell = plugin->shell;
-		
 		register_stock_icons (plugin);
-		
-		/* Add preferences */
-		gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog", NULL);
-		indent_button = glade_xml_get_widget (gxml, "set_indent_button");
-		g_signal_connect (G_OBJECT (indent_button), "clicked",
-						  G_CALLBACK (on_edit_editor_indent), plugin);
-		
-		anjuta_preferences_add_page (editor_plugin->prefs,
-									 gxml, "Documents", ICON_FILE);
-		anjuta_encodings_init (editor_plugin->prefs, gxml);
-		
-		indent_combo = glade_xml_get_widget (gxml, "pref_indent_style_combobox");
-		((DocmanPlugin*)plugin)->idt->pref_indent_combo = indent_combo;
-		g_signal_connect (G_OBJECT (indent_combo), "changed",
-						  G_CALLBACK (on_style_combo_changed), plugin);
-		
-		
-		indent_entry = glade_xml_get_widget (gxml, "preferences_style_entry");
-		((DocmanPlugin*)plugin)->idt->pref_indent_options = indent_entry;
-		((DocmanPlugin*)plugin)->idt->prefs = ((DocmanPlugin*)plugin)->prefs;
-		indent_init_load_style(((DocmanPlugin*)plugin)->idt);
-		
-		g_object_unref (G_OBJECT (gxml));
 	}
-	prefs_init (editor_plugin);
 	editor_plugin->action_groups = NULL;
 	/* Add all our editor actions */
 	for (i = 0; i < G_N_ELEMENTS (action_groups); i++)
@@ -1357,7 +1463,8 @@ activate_plugin (AnjutaPlugin *plugin)
 													_(action_groups[i].label),
 													action_groups[i].group,
 													action_groups[i].size,
-													GETTEXT_PACKAGE, plugin);
+													GETTEXT_PACKAGE, TRUE,
+													plugin);
 		editor_plugin->action_groups =
 			g_list_prepend (editor_plugin->action_groups, group);
 		actions = gtk_action_group_list_actions (group);
@@ -1378,7 +1485,7 @@ activate_plugin (AnjutaPlugin *plugin)
 												_(action_toggle_groups[i].label),
 												action_toggle_groups[i].group,
 												action_toggle_groups[i].size,
-												GETTEXT_PACKAGE, plugin);
+												GETTEXT_PACKAGE, TRUE, plugin);
 		editor_plugin->action_groups =
 			g_list_prepend (editor_plugin->action_groups, group);
 		actions = gtk_action_group_list_actions (group);
@@ -1404,7 +1511,8 @@ activate_plugin (AnjutaPlugin *plugin)
 	gtk_action_group_add_action (group, action);
 	
 	anjuta_ui_add_action_group (ui, "ActionGroupNavigation",
-								N_("Editor quick navigations"), group);
+								N_("Editor quick navigations"),
+								group, FALSE);
 	
 	/* Add UI */
 	editor_plugin->uiid = anjuta_ui_merge (ui, UI_FILE);
@@ -1412,13 +1520,6 @@ activate_plugin (AnjutaPlugin *plugin)
 							 "AnjutaDocumentManager", _("Documents"),
 							 "editor-plugin-icon",
 							 ANJUTA_SHELL_PLACEMENT_CENTER, NULL); 
-	/* Create Highlight submenu */
-	highlight_submenu = 
-		create_highlight_submenu (editor_plugin);
-	highlight_menu = gtk_ui_manager_get_widget (GTK_UI_MANAGER (ui),
-				"/MenuMain/MenuView/MenuViewEditor/MenuFormatStyle");
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (highlight_menu),
-							   highlight_submenu);
 	ui_states_init(plugin);
 	ui_give_shorter_names (plugin);
 	update_editor_ui (plugin, NULL);
@@ -1435,11 +1536,11 @@ activate_plugin (AnjutaPlugin *plugin)
 	/* Connect to save session */
 	g_signal_connect (G_OBJECT (plugin->shell), "save-session",
 					  G_CALLBACK (on_session_save), plugin);
+	/* Connect to save prompt */
+	g_signal_connect (G_OBJECT (plugin->shell), "save-prompt",
+					  G_CALLBACK (on_save_prompt), plugin);
 	
 	initialized = TRUE;
-
-	pref_set_style_combo(((DocmanPlugin*)plugin)->idt); 
-	
 	return TRUE;
 }
 
@@ -1454,11 +1555,13 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	
 	DEBUG_PRINT ("DocmanPlugin: Dectivating Editor plugin...");
 	
-	eplugin = (DocmanPlugin*)plugin;
+	eplugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 
 	prefs_finalize (eplugin);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
 										  G_CALLBACK (on_session_save), plugin);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
+										  G_CALLBACK (on_save_prompt), plugin);
 	
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	status = anjuta_shell_get_status (plugin->shell, NULL);
@@ -1484,7 +1587,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	node = eplugin->action_groups;
 	while (node)
 	{
-		GtkActionGroup *group = (GtkActionGroup*)node->data;
+		GtkActionGroup *group = GTK_ACTION_GROUP (node->data);
 		anjuta_ui_remove_action_group (ui, group);
 		node = g_list_next (node);
 	}
@@ -1504,14 +1607,14 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 dispose (GObject *obj)
 {
-	// DocmanPlugin *eplugin = (DocmanPlugin*)obj;
+	// DocmanPlugin *eplugin = ANJUTA_PLUGIN_DOCMAN (obj);
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
 }
 
 static void
 docman_plugin_instance_init (GObject *obj)
 {
-	DocmanPlugin *plugin = (DocmanPlugin*)obj;
+	DocmanPlugin *plugin = ANJUTA_PLUGIN_DOCMAN (obj);
 	plugin->uiid = 0;
 	plugin->g_tabbing = FALSE;
 	plugin->gconf_notify_ids = NULL;
@@ -1535,7 +1638,7 @@ static const gchar *
 ianjuta_docman_get_full_filename (IAnjutaDocumentManager *plugin,
 		const gchar *file, GError **e)
 {
-	AnjutaDocman *docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	AnjutaDocman *docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	return anjuta_docman_get_full_filename (ANJUTA_DOCMAN (docman), file);
 }
 
@@ -1543,14 +1646,14 @@ static IAnjutaEditor*
 ianjuta_docman_find_editor_with_path (IAnjutaDocumentManager *plugin,
 		const gchar *file_path, GError **e)
 {
-	AnjutaDocman *docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	AnjutaDocman *docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	return anjuta_docman_find_editor_with_path (ANJUTA_DOCMAN (docman), file_path);
 }
 
 static IAnjutaEditor*
 ianjuta_docman_get_current_editor (IAnjutaDocumentManager *plugin, GError **e)
 {
-	AnjutaDocman *docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	AnjutaDocman *docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	return anjuta_docman_get_current_editor (ANJUTA_DOCMAN (docman));
 }
 
@@ -1558,7 +1661,7 @@ static void
 ianjuta_docman_set_current_editor (IAnjutaDocumentManager *plugin,
 								   IAnjutaEditor *editor, GError **e)
 {
-	AnjutaDocman *docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	AnjutaDocman *docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	anjuta_docman_set_current_editor (ANJUTA_DOCMAN (docman),
 									  editor);
 }
@@ -1568,7 +1671,7 @@ ianjuta_docman_get_editors (IAnjutaDocumentManager *plugin, GError **e)
 {
 	AnjutaDocman *docman;
 	GList * editors = NULL;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	editors = anjuta_docman_get_all_editors (docman);
 	return editors;
 }
@@ -1578,7 +1681,7 @@ ianjuta_docman_goto_file_line (IAnjutaDocumentManager *plugin,
 							   const gchar *uri, gint linenum, GError **e)
 {
 	AnjutaDocman *docman;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	anjuta_docman_goto_file_line (docman, uri, linenum);
 }
 
@@ -1587,7 +1690,7 @@ ianjuta_docman_goto_file_line_mark (IAnjutaDocumentManager *plugin,
 		const gchar *uri, gint linenum, gboolean mark, GError **e)
 {
 	AnjutaDocman *docman;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	anjuta_docman_goto_file_line_mark (docman, uri, linenum, mark);
 }
 
@@ -1598,7 +1701,7 @@ ianjuta_docman_add_buffer (IAnjutaDocumentManager *plugin,
 {
 	AnjutaDocman *docman;
 	IAnjutaEditor *te;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	te = anjuta_docman_add_editor (docman, NULL, filename);
 	if (te)
 	{
@@ -1610,6 +1713,26 @@ ianjuta_docman_add_buffer (IAnjutaDocumentManager *plugin,
 	return NULL;
 }
 
+static gboolean
+ianjuta_docman_remove_buffer (IAnjutaDocumentManager *plugin,
+							  IAnjutaEditor *editor,
+							  gboolean save_before, GError **e)
+{
+	gint ret_val = TRUE;
+	AnjutaDocman *docman;
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
+	
+	if (save_before)
+	{
+		ret_val = anjuta_docman_save_editor (docman, editor,
+								 GTK_WIDGET (ANJUTA_PLUGIN (plugin)->shell));
+	}
+	if (ret_val)
+		anjuta_docman_remove_editor (docman, editor);
+	
+	return ret_val;
+}
+
 static void
 ianjuta_document_manager_iface_init (IAnjutaDocumentManagerIface *iface)
 {
@@ -1621,6 +1744,7 @@ ianjuta_document_manager_iface_init (IAnjutaDocumentManagerIface *iface)
 	iface->goto_file_line = ianjuta_docman_goto_file_line;
 	iface->goto_file_line_mark = ianjuta_docman_goto_file_line_mark;
 	iface->add_buffer = ianjuta_docman_add_buffer;
+	iface->remove_buffer = ianjuta_docman_remove_buffer;
 }
 
 /* Implement IAnjutaFile interface */
@@ -1629,7 +1753,7 @@ ifile_open (IAnjutaFile* plugin, const gchar* uri, GError** e)
 {
 	AnjutaDocman *docman;
 	
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	anjuta_docman_goto_file_line (docman, uri, -1);
 }
 
@@ -1638,7 +1762,7 @@ ifile_get_uri (IAnjutaFile* plugin, GError** e)
 {
 	AnjutaDocman *docman;
 	IAnjutaEditor* editor;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	editor = anjuta_docman_get_current_editor (docman);
 	if (editor != NULL)
 		return g_strdup (ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL));
@@ -1663,7 +1787,7 @@ isaveable_save (IAnjutaFileSavable* plugin, GError** e)
 	AnjutaDocman *docman;
 	IAnjutaEditor* editor;
 	GList* editors;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	editors = anjuta_docman_get_all_editors(docman);
 	while(editors)
 	{
@@ -1692,7 +1816,7 @@ isavable_is_dirty(IAnjutaFileSavable* plugin, GError** e)
 	IAnjutaEditor* editor;
 	GList* editors;
 	gboolean retval = FALSE;
-	docman = ANJUTA_DOCMAN ((((DocmanPlugin*)plugin)->docman));
+	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	editors = anjuta_docman_get_all_editors(docman);
 	while(editors)
 	{
@@ -1723,10 +1847,63 @@ isavable_iface_init (IAnjutaFileSavableIface *iface)
 	iface->is_dirty = isavable_is_dirty;
 }
 
+static void
+ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	GtkWidget *indent_button;
+	GtkWidget *indent_combo;
+	GtkWidget *indent_entry;
+	GladeXML* gxml;
+		
+	AnjutaPlugin* plugin = ANJUTA_PLUGIN(ipref);
+		
+	/* Add preferences */
+	gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog", NULL);
+	indent_button = glade_xml_get_widget (gxml, "set_indent_button");
+	g_signal_connect (G_OBJECT (indent_button), "clicked",
+						  G_CALLBACK (on_edit_editor_indent), plugin);
+		
+	anjuta_preferences_add_page (prefs,
+									gxml, "Documents", _("Documents"),  ICON_FILE);
+	anjuta_encodings_init (prefs, gxml);
+		
+	indent_combo = glade_xml_get_widget (gxml, "pref_indent_style_combobox");
+	ANJUTA_PLUGIN_DOCMAN (plugin)->idt->pref_indent_combo = indent_combo;
+	g_signal_connect (G_OBJECT (indent_combo), "changed",
+						  G_CALLBACK (on_style_combo_changed), plugin);
+		
+		
+	indent_entry = glade_xml_get_widget (gxml, "preferences_style_entry");
+	ANJUTA_PLUGIN_DOCMAN (plugin)->idt->pref_indent_options = indent_entry;
+	ANJUTA_PLUGIN_DOCMAN (plugin)->idt->prefs = ANJUTA_PLUGIN_DOCMAN (plugin)->prefs;
+	indent_init_load_style(ANJUTA_PLUGIN_DOCMAN (plugin)->idt);
+		
+	g_object_unref (G_OBJECT (gxml));
+	prefs_init(ANJUTA_PLUGIN_DOCMAN (plugin));
+	pref_set_style_combo(ANJUTA_PLUGIN_DOCMAN (plugin)->idt); 
+}
+
+static void
+ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	DocmanPlugin* plugin = ANJUTA_PLUGIN_DOCMAN (ipref);
+	prefs_finalize(plugin);
+	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG(prefs), 
+		_("Documents"));
+}
+
+static void
+ipreferences_iface_init(IAnjutaPreferencesIface* iface)
+{
+	iface->merge = ipreferences_merge;
+	iface->unmerge = ipreferences_unmerge;	
+}
+
 ANJUTA_PLUGIN_BEGIN (DocmanPlugin, docman_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE(ianjuta_document_manager, IANJUTA_TYPE_DOCUMENT_MANAGER);
 ANJUTA_PLUGIN_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
 ANJUTA_PLUGIN_ADD_INTERFACE(isavable, IANJUTA_TYPE_FILE_SAVABLE);
+ANJUTA_PLUGIN_ADD_INTERFACE(ipreferences, IANJUTA_TYPE_PREFERENCES);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (DocmanPlugin, docman_plugin);
