@@ -25,6 +25,14 @@
 
 #define ICON_FILE "anjuta-patch-plugin.png"
 
+extern GType patch_plugin_get_type (GluePlugin *plugin);
+#define ANJUTA_TYPE_PLUGIN_PATCH         (patch_plugin_get_type (NULL))
+#define ANJUTA_PLUGIN_PATCH(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_TYPE_PLUGIN_PATCH, PatchPlugin))
+#define ANJUTA_PLUGIN_PATCH_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), ANJUTA_TYPE_PLUGIN_PATCH, PatchPluginClass))
+#define ANJUTA_IS_PLUGIN_PATCH(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), ANJUTA_TYPE_PLUGIN_PATCH))
+#define ANJUTA_IS_PLUGIN_PATCH_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), ANJUTA_TYPE_PLUGIN_PATCH))
+#define ANJUTA_PLUGIN_PATCH_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), ANJUTA_TYPE_PLUGIN_PATCH, PatchPluginClass))
+
 typedef struct _PatchPlugin PatchPlugin;
 typedef struct _PatchPluginClass PatchPluginClass;
 
@@ -37,8 +45,14 @@ struct _PatchPlugin {
 	GtkWidget* file_chooser;
 	GtkWidget* patch_chooser;
 	GtkWidget* dialog;
+	GtkWidget* output_label;
+	GtkWidget* patch_button;
+	GtkWidget* cancel_button;
+	GtkWidget* dry_run_check;
 	GladeXML* gxml;
 	
+
+	gboolean executing;
 	gint uiid;
 };
 
